"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseApplication = void 0;
const events_1 = __importDefault(require("events"));
const applicationEvents_1 = require("./applicationEvents");
const applicationNotInitialisedError_1 = require("./applicationNotInitialisedError");
class BaseApplication {
    constructor(app) {
        this.app = app;
        this.currentRequest = null;
        this.setCurrentRequest = (currentRequest) => {
            this.currentRequest = currentRequest;
        };
        this.getCurrentRequest = () => {
            if (!this.currentRequest) {
                throw new Error(`Request has not been initialised.`);
            }
            return this.currentRequest;
        };
        this.setUp = async () => {
            this.initialiseEventEmitter();
            await this.initialise();
            await this.initialisePreMiddlewares();
            BaseApplication.getEventEmitter().emit(applicationEvents_1.ApplicationEvents.beforeRoutesRegistered, this.app);
            await this.registerRoutes();
            await this.initialisePostMiddlewares();
        };
        this.initialiseEventEmitter = () => {
            if (BaseApplication.eventEmitter) {
                return;
            }
            BaseApplication.eventEmitter = new events_1.default();
        };
    }
}
exports.BaseApplication = BaseApplication;
BaseApplication.init = (application) => {
    BaseApplication.instance = application;
    return BaseApplication.instance;
};
BaseApplication.getInstance = () => {
    if (!BaseApplication.instance) {
        throw new applicationNotInitialisedError_1.ApplicationNotInitialisedError();
    }
    return BaseApplication.instance;
};
BaseApplication.getEventEmitter = () => {
    if (!BaseApplication.eventEmitter) {
        throw new Error('Application event emitter has not been initialised yet.');
    }
    return BaseApplication.eventEmitter;
};
/**
 * ! we encourage you not to invoke this function unless you have a deep understanding of how MEN MVC framework works
 */
BaseApplication.clearEventEmitter = () => {
    BaseApplication.eventEmitter = null;
};
/**
 * ! we encourage you not to invoke this function unless you have a deep understanding of how MEN MVC framework works
 */
BaseApplication.clearInstance = () => {
    BaseApplication.instance = null;
};
